from flask import Flask, jsonify
from flask import abort
from flask import make_response
from flask import request
from pymongo import MongoClient
from flask_cors import CORS

app = Flask(__name__)
CORS(app)

@app.route('/classroom/buildings/', methods=['GET'])
def get_buildings():
  myclient = MongoClient()
  mydb = myclient["classroomDB"]
  buildings = mydb["buildings"]
  bs = buildings.find({},{"bcode": 1, "bname": 1, "_id": 0})
  bldgs = []
  for b in bs:
    bldgs.append({'bldg':b["bcode"], 'bname':b["bname"]})
  result = {'buildings': bldgs}
  return jsonify(result)

@app.route('/classroom/buildings/', methods=['POST'])
def insert_buildings():
  bcode = request.json['bcode']
  bname = request.json['bname']
  myclient = MongoClient()
  mydb = myclient["classroomDB"]
  buildings = mydb["buildings"]
  bs = list(buildings.find({"bcode": bcode},{"bcode": 1, "bname": 1, "_id": 0}))
  # check if building already present. could have defined a unique index on bcode
  # to enforce primary key!!
  if len(bs) == 1:
    result = {"ok": False, "bcode": bcode, "bname": bname}
  else:
    buildings.insert_one({"bcode": bcode, "bname": bname})
    result = {"ok": True, "bcode": bcode, "bname": bname}
  return jsonify(result)

@app.route('/classroom/rooms/<string:bldg>/', methods=['GET'])
def get_rooms_in_building(bldg):
  myclient = MongoClient()
  mydb = myclient["classroomDB"]
  rooms = mydb["rooms"]
  rs = rooms.find({"bcode": bldg},{"rnumber": 1, "_id": 0})
  rooms = []
  for r in rs:
    rooms.append({'rnumber':r["rnumber"]})
  result = {'rooms': rooms}
  return jsonify(result)

@app.route('/classroom/rooms/<string:bldg>/<string:rnumber>/', methods=['GET'])
def get_room_details(bldg,rnumber):
  myclient = MongoClient()
  mydb = myclient["classroomDB"]
  buildings = mydb["buildings"]
  rooms = mydb["rooms"]
  bname = list(buildings.find({"bcode": bldg},{"bname": 1, "_id": 0}))[0]["bname"]
  rs = list(rooms.find({"bcode": bldg, "rnumber": rnumber},
                       {"cap": 1, "layout": 1, "type": 1, "dept": 1, "_id": 0}))
  if len(rs) == 0:
      abort(404)
  result = {
    'building' : bname+" ("+bldg+")",
    'rnumber' : rnumber,
    'cap': rs[0]['cap'],
    'layout': rs[0]['layout'],
    'type': rs[0]['type'],
    'dept': rs[0]['dept']
  }
  return jsonify(result)

@app.route('/classroom/rooms/<string:bldg>/<string:rnumber>/media/', methods=['GET'])
def get_room_media(bldg,rnumber):
  myclient = MongoClient()
  mydb = myclient["classroomDB"]
  rooms = mydb["rooms"]
  media = mydb["media"]
  rs = list(rooms.find({"bcode": bldg, "rnumber": rnumber},
                       {"media": 1, "_id": 0}))
  if 'media' in rs[0]:
    ms = []
    for m in rs[0]['media']:
      mdesc = media.find({"mcode": m},{"description": 1, "_id": 0})[0]
      ms.append({'mcode': m, 'mdescription': mdesc['description']})
    result = {'media': ms}
  else:
    result = {'media': []}
  return jsonify(result)

@app.route('/classroom/media/', methods=['GET'])
def get_media():
  myclient = MongoClient()
  mydb = myclient["classroomDB"]
  media = mydb["media"]
  ms = media.find({},{"mcode": 1, "description": 1, "_id": 0})
  result = []
  for m in ms:
    result.append({'mcode':m["mcode"], 'mdescription':m["description"]})
  result = {'media': result}
  return jsonify(result)

@app.route('/classroom/<string:bldg>/<string:rnumber>/', methods=['PUT'])
def update_room_cap(bldg,rnumber):
  myclient = MongoClient()
  mydb = myclient["classroomDB"]
  rooms = mydb["rooms"]
  rs = list(rooms.find({"bcode": bldg, "rnumber": rnumber},{"_id": 1}))
  # check if room present.
  if len(rs) == 1:
    rooms.update_one({"bcode": bldg, "rnumber": rnumber}, 
                     {"$set": {"cap": request.json['newcap']}, 
                      "$currentDate": { "lastModified": True } })
    result = {"ok": True, "bldg": bldg, "rnumber":rnumber, "cap":request.json['newcap']}
  else:
    result = {"ok": False, "bldg": bldg, "rnumber":rnumber, "cap":request.json['newcap']}
  return jsonify(result)

@app.route('/classroom/rooms/<string:bldg>/<string:rnumber>/', methods=['POST'])
def insert_room(bldg,rnumber):
  cap = request.json['cap']
  layout = request.json['layout']
  rtype = request.json['rtype']
  dept = request.json['dept']
  myclient = MongoClient()
  mydb = myclient["classroomDB"]
  rooms = mydb["rooms"]
  rs = list(rooms.find({"bcode": bldg, "rnumber": rnumber},{"bcode": 1, "_id": 0}))
  # check if room already present. could have defined a unique index on bcode
  # to enforce primary key!!
  if len(rs) == 1:
    result = {"ok": False, "bldg": bldg, "rnumber":rnumber}
  else:
    media = request.json['media']
    ms = []
    for m in media:
      ms.append(m) 
    rooms.insert_one({"bcode": bldg, "rnumber": rnumber, "cap": cap, "layout": layout,
                      "type": rtype, "dept": dept, "media": ms})
    result = {"ok": True, "bldg": bldg, "rnumber":rnumber}
  return jsonify(result)

@app.route('/classroom/rooms/<string:bldg>/<string:rnumber>/', methods=['DELETE'])
def delete_room(bldg,rnumber):
  myclient = MongoClient()
  mydb = myclient["classroomDB"]
  rooms = mydb["rooms"]
  rs = list(rooms.find({"bcode": bldg, "rnumber": rnumber},{"_id": 1}))
  # check if room present. 
  if len(rs) == 1:
    rooms.delete_one({"bcode": bldg, "rnumber": rnumber})
    result = {"ok": True, "bldg": bldg, "rnumber":rnumber}
  else:
    result = {"ok": False, "bldg": bldg, "rnumber":rnumber}
  return jsonify(result)

@app.errorhandler(404)
def not_found(error):
    return make_response(jsonify({'error': 'Not found'}), 404)

if __name__ == '__main__':
    app.run(host='localhost',debug=True)
